from woodchipper.context import LoggingContext


class WoodchipperLambda:
    def __init__(self, app):
        self._app = app

    def __call__(self, environ, start_response):
        if "LAMBDA_TASK_ROOT" not in environ:
            # This is a decent sentinel for being in a Lambda environment
            return self._app(environ, start_response)
        with LoggingContext(
            **{"aws-request-id": getattr(environ.get("lambda.context", object()), "aws_request_id", None)},
            _prefix="lambda",
        ):
            return self._app(environ, start_response)



import structlog

import woodchipper.processors
from woodchipper import BaseConfigClass

callsite_parameters = [
    structlog.processors.CallsiteParameter.FUNC_NAME,
    structlog.processors.CallsiteParameter.LINENO,
    structlog.processors.CallsiteParameter.MODULE,
]


class Minimal(BaseConfigClass):
    """
    Really used for unit tests. That's it.
    """

    processors = [
        structlog.stdlib.add_log_level,
    ]
    factory = structlog.stdlib.LoggerFactory()
    wrapper_class = structlog.stdlib.BoundLogger
    renderer = structlog.processors.JSONRenderer()


class DevLogToStdout(BaseConfigClass):
    processors = [
        structlog.stdlib.add_log_level,
        structlog.stdlib.add_logger_name,
        structlog.stdlib.ExtraAdder(),
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.StackInfoRenderer(),
        structlog.dev.set_exc_info,
        structlog.processors.UnicodeDecoder(),
        structlog.processors.TimeStamper(fmt="%Y-%m-%d %H:%M.%S", utc=False),
        structlog.processors.CallsiteParameterAdder(parameters=callsite_parameters),
        woodchipper.processors.inject_context_processor,
    ]
    factory = structlog.stdlib.LoggerFactory()
    wrapper_class = structlog.stdlib.BoundLogger
    renderer = structlog.dev.ConsoleRenderer()


class JSONLogToStdout(BaseConfigClass):
    processors = [
        structlog.stdlib.add_log_level,
        structlog.stdlib.add_logger_name,
        structlog.stdlib.ExtraAdder(),
        structlog.stdlib.PositionalArgumentsFormatter(),
        structlog.processors.StackInfoRenderer(),
        woodchipper.processors.GitVersionProcessor(),
        structlog.processors.TimeStamper(fmt="%Y-%m-%d %H:%M.%S", utc=False),
        structlog.processors.CallsiteParameterAdder(parameters=callsite_parameters),
        structlog.processors.format_exc_info,
        structlog.processors.UnicodeDecoder(),
        woodchipper.processors.inject_context_processor,
    ]
    factory = structlog.stdlib.LoggerFactory()
    wrapper_class = structlog.stdlib.BoundLogger
    renderer = structlog.processors.JSONRenderer()



import uuid

from flask import g, request

from woodchipper.context import LoggingContext

BLACKLISTED_HEADERS = ["authorization", "cookie"]


class WoodchipperFlask:
    def __init__(self, app, blacklisted_headers=BLACKLISTED_HEADERS, request_id_factory=None):
        self._app = app
        self._blacklisted_headers = blacklisted_headers
        self._request_id_factory = request_id_factory or (lambda: str(uuid.uuid4()))
        self.vanilla_full_dispatch_request = app.full_dispatch_request

    def wrapped_full_dispatch_request(self):
        if not getattr(g, "request_id", None):
            g.request_id = self._request_id_factory()
        with LoggingContext(
            **{
                "id": g.request_id,
                "body_size": request.content_length,
                "method": request.method,
                "url": request.url,
                **{
                    f"header.{header.lower()}": ("******" if header.lower() in self._blacklisted_headers else value)
                    for header, value in request.headers.items()
                },
            },
            _prefix="http",
        ):
            return self.vanilla_full_dispatch_request()

    def chipperize(self):
        self._app.full_dispatch_request = self.wrapped_full_dispatch_request
